@@ -1,7 +1,6 @@
-from hypernetx.exception import HyperNetXException, HyperNetXError, HyperNetXNotImplementedError
+from hypernetx.exception import HyperNetXException, HyperNetXError
 from hypernetx.read_write import to_pickle, load_from_pickle
 from hypernetx.classes import *
 from hypernetx.reports import *
 from hypernetx.drawing import *
-from hypernetx.algorithms import *
-from hypernetx.utils import *
+from hypernetx.algorithms import *
\ No newline at end of file
